#include "IZ_app.h"

IZ_ProcedureResult IZ_InitializeApp(IZ_App* app) {
	if (SDL_Init(
		SDL_INIT_VIDEO
		| SDL_INIT_GAMECONTROLLER
		| SDL_INIT_EVENTS
	) < 0) {
		fprintf_s(stderr, "SDL could not initialize! SDL_Error: %s\n", SDL_GetError());
		return 1;
	}

	char config_path[128];
	IZ_GetConfigPath(config_path, 128);

	if (IZ_InitializeVideo(config_path, &app->video_state)) {
		return 2;
	}

	IZ_InitializeInput(config_path, &app->input_state);

	app->quit = false;
	return 0;
}

void IZ_TeardownApp(IZ_App* app) {
	IZ_TeardownInput(&app->input_state);
	IZ_TeardownVideo(&app->video_state);
	SDL_Quit();
}

void IZ_HandleSDLEvents(IZ_App* app) {
	while (SDL_PollEvent(&app->input_state.sdl_event) != 0) {
		if (app->input_state.sdl_event.type == SDL_QUIT) {
			app->quit = true;
			break;
		}

		IZ_HandleSDLInputEvents(app->input_state.sdl_event, &app->input_state);
	}
}

void IZ_HandlePortMIDIEvents(IZ_App* app) {
	for (uint8_t player_index = 0; player_index < PLAYERS; player_index += 1) {
		int32_t midi_events_count = Pm_Read(
			app->input_state.midi_input_state[player_index].stream,
			app->input_state.midi_input_state[player_index].event_buffer,
			1024
		);

		if (midi_events_count > 0) {
			for (int32_t midi_event_index = 0; midi_event_index < midi_events_count; midi_event_index += 1) {
				IZ_HandlePortMIDIInputEvents(
					app->input_state.midi_input_state->event_buffer[midi_event_index],
					&app->input_state
				);
			}
		}
	}
}

void IZ_HandleEvents(IZ_App* app) {
	IZ_HandleSDLEvents(app);
	IZ_HandlePortMIDIEvents(app);
}

IZ_ProcedureResult IZ_RunApp(IZ_App* app, uint8_t argc, char* argv[]) {
	printf_s("Args (%u):\n", argc);
	for (uint8_t i = 0; i < argc; i += 1) {
		printf_s("  %s", argv[i]);
	}

	IZ_ProcedureResult init_result = IZ_InitializeApp(app);
	if (init_result) {
		return init_result;
	}

	while (true) {
		uint64_t ticks = SDL_GetTicks64();

		// TODO do audio processing
		// TODO do networking?

		// process events
		IZ_HandleEvents(app);
		if (app->quit) {
			break;
		}

		IZ_UpdateVideo(&app->video_state, &app->input_state, ticks);
	}

	IZ_TeardownApp(app);
	return 0;
}
