#include "IZ_video.h"

IZ_ProcedureResult IZ_SaveVideoConfig(const char* config_path, IZ_VideoConfig* config) {
	if (!ini_putl("Video", "Width", config->width, config_path)) {
		return 1;
	}
	if (!ini_putl("Video", "Height", config->height, config_path)) {
		return 1;
	}
	if (!ini_putl("Video", "MaxFps", config->max_fps, config_path)) {
		return 1;
	}

	return 0;
}

void IZ_LoadVideoConfig(const char* config_path, IZ_VideoConfig* config) {
	config->width = ini_getl("Video", "Width", IZ_DEFAULT_VIDEO_STATE.config.width, config_path);
	config->height = ini_getl("Video", "Height", IZ_DEFAULT_VIDEO_STATE.config.height, config_path);
	config->max_fps = ini_getl("Video", "MaxFps", IZ_DEFAULT_VIDEO_STATE.config.max_fps, config_path);
}

IZ_ProcedureResult IZ_InitializeVideo(const char* config_path, IZ_VideoState* state) {
	SDL_memcpy(state, &IZ_DEFAULT_VIDEO_STATE, sizeof(IZ_VideoState));

	IZ_LoadVideoConfig(config_path, &state->config);
	if (IZ_SaveVideoConfig(config_path, &state->config)) {
		fprintf_s(stderr, "Error committing video config.\n");
	}
	state->last_update_at = 0u;

	SDL_Window* window = SDL_CreateWindow(
		APP_NAME,
		SDL_WINDOWPOS_CENTERED,
		SDL_WINDOWPOS_CENTERED,
		state->config.width,
		state->config.height,
		SDL_WINDOW_SHOWN
	);
	if (window == NULL) {
		fprintf_s(stderr, "Window could not be created! SDL_Error: %s\n", SDL_GetError());
		return 1;
	}
	state->window = window;
	state->renderer = SDL_CreateRenderer(window, -1, SDL_RENDERER_ACCELERATED);

	return 0;
}

void IZ_UpdateVideo(IZ_VideoState* video_state, IZ_InputState* input_states, uint64_t ticks) {
	if (ticks - video_state->last_update_at > 1000 / video_state->config.max_fps) {
		// Update window
		SDL_SetRenderDrawColor(video_state->renderer, 0x00, 0x00, 0x00, 0xff);
		SDL_RenderClear(video_state->renderer);

		SDL_SetRenderDrawColor(video_state->renderer, 0x00, 0xff, 0xff, 0xff);
		uint64_t the_ticks = ticks;
		for (uint8_t i = 0; i < 64; i += 1) {
			const uint8_t column = i % 32;
			const uint8_t row = i / 32;
			const uint8_t size = 4;

			if (the_ticks & 0x1) {
				SDL_RenderFillRectF(video_state->renderer, &(SDL_FRect) {
					 video_state->config.width - ((column + 1) * size),
					video_state->config.height - ((row + 1) * size),
					size,
					size
				});
			}
			the_ticks >>= 1;
		}

		SDL_SetRenderDrawColor(video_state->renderer, 0xff, 0xff, 0x00, 0xff);
		// TODO refer to app's state
		for (uint8_t p = 0; p < PLAYERS; p += 1) {
			for (uint8_t i = 0; i < CONTROLS; i += 1) {
				const uint8_t column = (i % 4) + (p * 4);
				const uint8_t row = i / 4;
				const IZ_Action bitflag = (0x1 << i);
				const uint8_t size = 4;
				if (input_states->action[p] & bitflag) {
					SDL_RenderFillRectF(video_state->renderer, &(SDL_FRect) {
						column * size,
						row * size,
						size,
						size
					});
				}
			}
		}

		SDL_RenderPresent(video_state->renderer);
		video_state->last_update_at = ticks;
	}
}

void IZ_TeardownVideo(IZ_VideoState* state) {
	SDL_DestroyWindow(state->window);
}
